import prompts from 'prompts';
export class Form {
    constructor(questions, answers = {}) {
        this.questions = questions.filter(q => !q.argumentOnly);
        prompts.override(answers);
        this.applyCustomizations();
    }
    applyCustomizations() {
        this.handleWhenFunction();
        this.handleStringChoices();
        this.handleSuggestOnly();
    }
    handleWhenFunction() {
        this.questions.filter(q => typeof q.when === 'function').forEach(q => {
            const stringType = q.type.toString();
            q.type = (prev, values) => !!q.when(values) ? stringType : null;
        });
    }
    handleStringChoices() {
        this.questions.filter(q => Array.isArray(q.choices)).forEach(q => {
            q.choices = q.choices.map(c => typeof c === 'string' ? { title: c } : c);
        });
    }
    handleSuggestOnly() {
        this.questions.filter(q => q.type === 'autocomplete' && !!q.suggestOnly).forEach(q => {
            // implementation from this issue https://github.com/terkelg/prompts/issues/131
            q.onState = function () {
                this.fallback = { title: this.input };
                // Check to make sure there are no suggestions so we do not override a suggestion
                if (this.suggestions.length === 0) {
                    this.value = this.input;
                }
            };
        });
    }
    async run() {
        // this.generateAutocompleteSource(formQuestions)
        return await prompts(this.questions);
    }
    inject(answers) {
        prompts.inject(answers);
    }
}
